==> stam_1.4.0.tar.gz: stam/R/09_server.R <==
################################################################################
#
#   09_server.R
#
#   Waits until it finds a tab-delimited datafile written by a
#   CGI-script in the temporary path. Performs the demanded action
#   according to the entries in the table. Form of the table:
#
#   projectPath	<path>
#   action	redraw_pred_img|refit_model
#   ... (further tab delimited parameters)
#
#   As soon as the server gets busy, it writes a corresponding
#   busy.html page into the project directory. When the task is
#   finished, the temporary table is removed and a redirecting
#   busy.html page is written. 
#
#   written by Claudio Lottaz (MPI-MG) in February-September 2004
#
################################################################################
#
#   Redraw prediction image
#

redraw.pred.img <- function(eval, data.path, aclass, show.graph, lab.len, sample.labels, 
                            f.red, f.green, f.blue, t.red, t.green, t.blue, 
                            minspec, minsens, maxsens, bin.thresh) {
  cat("    . redrawing prediction image...\n")
  write.busy(data.path, "Redrawing prediction images")
  show.graph <- ifelse(show.graph=="ON", T, F)
  sample.labels <- ifelse(sample.labels=="ON", T, F)
  filenamestem <- paste(data.path, "index", sep="")
  if (bin.thresh == "-") binthresh <- NULL
  else binthresh <- as.numeric(bin.thresh)
  time.stamp <- format(Sys.time(), "%j%H%M%S")

  file.remove(list.files(data.path, "index_pred_img[0-9]*.p", full.names=T))
  map <- image(eval@pred, outfile=filenamestem, aclass=aclass, 
               show.graph=show.graph,max.label.length=lab.len,sample.labels=sample.labels,
               col=stam.rgb.colors(r0=f.red, g0=f.green, b0=f.blue, 
                                   r1=t.red, g1=t.green, b1=t.blue), 
               minspec=minspec, minsens=minsens, maxsens=maxsens, bin.thresh=binthresh)
  file.rename(paste(data.path, "index_pred_img.png", sep=""),
              paste(data.path, "index_pred_img", time.stamp, ".png", sep=""))
  image(eval@pred, outfile=filenamestem, aclass=aclass, 
        show.graph=show.graph,max.label.length=lab.len, sample.labels=sample.labels, ps=T,
        col=stam.rgb.colors(r0=f.red, g0=f.green, b0=f.blue, 
                            r1=t.red, g1=t.green, b1=t.blue), 
        minspec=minspec, minsens=minsens, maxsens=maxsens, bin.thresh=binthresh)
  file.rename(paste(data.path, "index_pred_img.ps", sep=""),
              paste(data.path, "index_pred_img", time.stamp, ".ps", sep=""))

  lines <- readLines(paste(filenamestem, ".html", sep=""))
  lines[1] <- gsub("index_pred_img[0-9]*.p",
                   paste("index_pred_img", time.stamp, ".p", sep=""),
                   lines[1])
  lines[2] <- paste(MAP(name="pred_map", map),
                    stam.pred.img.form(filenamestem, aclass, 
                                       minspec, minsens, maxsens, bin.thresh,
                                       f.red, f.green, f.blue, t.red, t.green, t.blue,
                                       show.graph, lab.len, sample.labels), sep="")
  cat(lines, sep="\n", file=paste(filenamestem, ".html", sep=""))
}

################################################################################
#
#   Refit the model with new alpha
#

refit.model <- function(eval, data.path, aclass, delta, collapse.scnodes) {
  cat("    . refitting modes...\n")
  sink("tmp.log")
  write.busy(data.path, "Refitting model")
  collapse.scnodes <- ifelse(collapse.scnodes == "ON", TRUE, FALSE) 
  trainingset <- eval@cv@sample.labels
  fit <- stam.fit(eval@cv, eval@exprs[,trainingset], alpha=eval@fit@alpha, 
                  delta=delta, collapse.scnodes=collapse.scnodes)
  eval@fit <<- fit

  write.busy(data.path, "Predicting testset")
  pred <- stam.predict(fit, eval@exprs, eval@pred@cls, eval@testset)
  eval@pred <<- pred

  write.busy(data.path, "Replacing results")
  save(eval, file=paste(data.path, "eval.RData", sep=""), compress=T)
  filename <- paste(data.path, "index.html", sep="")
  file.remove(list.files(data.path, "index*", full.names=T))
  file.remove(list.files(data.path, "GO*", full.names=T))
  stam.writeHTML(eval, outfile=data.path, aclass = aclass)
  # avoid browser cache by using time stamps in figures
  time.stamp <- format(Sys.time(), "%j%H%M%S")
  for (f in list.files(data.path, "*.png", full.names=T))
    file.rename(f, sub(".png", paste(time.stamp, ".png", sep=""), f))
  for (f in list.files(data.path, "*.ps", full.names=T))
    file.rename(f, sub(".ps", paste(time.stamp, ".ps", sep=""), f))
  buffer <- readLines(filename)
  buffer[1] <- gsub(".png'>", paste(time.stamp,".png'>",sep=""), buffer[1])
  buffer[1] <- gsub('.ps">Down', paste(time.stamp,'.ps">Down',sep=""), buffer[1])
  cat(buffer, sep="\n", file=filename)
  sink()
  file.remove("tmp.log")
}

################################################################################
#
#   Auxiliaries
#

busy.buffer <- ""
write.busy <- function(data.path, add.item = NULL) {
  if (busy.buffer == "") {
    busy.buffer <<- paste("<HTML><HEAD><TITLE>Working...</TITLE>",
                         "<LINK REL='stylesheet' HREF='/includes/style.css' TYPE='text/css'>",
                         "</HEAD><BODY ONLOAD='setTimeout(\"document.location.reload(true)\", 2000)'><H1>Working...</H1></BODY></HTML>", sep="")
  }

  if (!is.null(add.item)) {
    busy.buffer <<- sub("\\.\\.\\.</LI>", paste("</LI><LI>",add.item,"...</LI>",sep=""), 
                        busy.buffer)
    busy.buffer <<- sub("</H1></BODY>", 
                        paste("</H1><UL><LI>", add.item, "...</LI></UL></BODY>", sep=""),
                        busy.buffer)
  }

  cat(busy.buffer, file=paste(data.path, "busy.html", sep=""))
}

write.redirect <- function(data.path, to.URL) {
  busy.buffer <<- ""
  filename <- paste(data.path, "busy.html", sep="")
  cat("<HTML><HEAD></HEAD><BODY ONLOAD='document.hiddenform.submit()'>",
      "<FORM NAME='hiddenform' action='/cgi-bin/stam/rmbusy.pl'>",
      "<INPUT TYPE=HIDDEN NAME=toURL VALUE='", to.URL, "'>",
      "<INPUT TYPE=HIDDEN NAME=datapath VALUE='", data.path, "'>",
      "</FORM></BODY></HTML>", sep="", file=filename)
}

################################################################################
#
#   Server loop
#

stam.serve <- function(tmp.path = NULL, cgi.path = NULL, cgi.url = NULL) {
  # modify installation if needed or required
  old.cgi.url <- stam.cgi.url
  stamenv <- as.environment(match("package:stam", search()))
  if (!is.null(tmp.path)) assign("stam.tmp.path", tmp.path, stamenv)
  if (!is.null(cgi.path)) assign("stam.cgi.path", cgi.path, stamenv)
  if (!is.null(cgi.url))  assign("stam.cgi.url",  cgi.url,  stamenv)
  if (stam.tmp.path == "")
    stam.tmp.path <- readline(paste("Which is your temporary directory", 
                                    "with write permission for the WWW server?\n"))
  if (stam.cgi.path == "") 
    stam.cgi.path <- readline("Which is your CGI directory?\n")
  if (stam.cgi.url == "")
    stam.cgi.url <- readline("Which is the URL to the CGI directory?\n")
  if (stam.tmp.path == "") stop("stam.tmp.path not provided.")
  if (stam.cgi.path == "") stop("stam.cgi.path not provided.")
  if (stam.cgi.url == "") stop("stam.cgi.url not provided.")
  data.file <- paste(help(package=stam)$path, "/data/server.config.rda", sep="")
  save(stam.tmp.path, stam.cgi.path, stam.cgi.url, file=data.file)
  if (old.cgi.url != stam.cgi.url) 
    cat("The CGI URL has changed, make sure to regenerate the HTML pages\n",
        "to be worked with through the server.\n", sep="")
  
  # write stam.tmp.path into CGI scripts
  cgi.srcdir <- paste(library(help=stam)$path, "cgi", sep="/")
  for (f in list.files(cgi.srcdir, "*.pl", full.names=TRUE)) {
    flines <- readLines(f)
    flines <- gsub("<<<STAM.TEMP.PATH>>>", stam.tmp.path, flines)
    cgi.name <- gsub(cgi.srcdir, stam.cgi.path, f)
    writeLines(flines, cgi.name)
  }

  # start listening
  data.path = "";
  options(stam.write.forms=TRUE)
  cat(" - start listening '", stam.tmp.path, "'...\n", 
      "   Stop the server by creating '", stam.tmp.path, "/quit'.\n",
      "   Make sure you have written the HTML pages to be worked with through the\n",
      "   StAM server with 'options(stam.write.forms=TRUE)'\n", sep="")
  while (TRUE) {
    file.list <- list.files(stam.tmp.path, "quit", full.names=T)
    qfile <- grep("quit$", file.list)
    if (length(qfile) > 0) {
      file.remove(file.list[qfile])
      break
    }
    file.list <- list.files(stam.tmp.path, "*tab", full.names=T)
    for (curr.file in file.list) {
      cat(" - treating", curr.file, "...\n")
      tab <- read.table(curr.file, row.names=1, sep="\t")
      x <- as.character(tab[,1])
      names(x) <- rownames(tab)
      if (data.path != "") write.busy(x["data.path"])
      if (data.path != x["data.path"]) {
        cat("    . loading data...\n")
        write.busy(data.path, "Loading data")
        data.path <- x["data.path"]
        load(paste(data.path, "eval.RData", sep=""))
      }
      if (x["action"] == "redraw_pred_img") 
        redraw.pred.img(eval, data.path, x["aclass"], x["show.graph"], 
                        as.numeric(x["lab.len"]), x["sample.labels"], 
                        as.numeric(x["f.red"]), as.numeric(x["f.green"]), 
                        as.numeric(x["f.blue"]), as.numeric(x["t.red"]), 
                        as.numeric(x["t.green"]), as.numeric(x["t.blue"]), 
                        as.numeric(x["minspec"]), as.numeric(x["minsens"]), 
                        as.numeric(x["maxsens"]), x["bin.thresh"])
  
      if (x["action"] == "refit_model") 
        refit.model(eval,data.path,x["aclass"],as.numeric(x["delta"]),x["collapse_scnodes"])
      if (x["action"] == "remove_busy.html") {
        cat("    . removeing ", data.path, "busy.html", "...\n", sep="")
        file.remove(paste(data.path, "busy.html", sep=""))
      } else write.redirect(data.path, x["to.URL"])
      cat("    . done\n")
      file.remove(curr.file)
    }
    Sys.sleep(1)
  }
}


#
#   End of File
#
################################################################################

==> stam_1.8.0.tar.gz: stam/R/09_server.R <==
################################################################################
#
#   09_server.R
#
#   Waits until it finds a tab-delimited datafile written by a
#   CGI-script in the temporary path. Performs the demanded action
#   according to the entries in the table. Form of the table:
#
#   projectPath	<path>
#   action	redraw_pred_img|refit_model
#   ... (further tab delimited parameters)
#
#   As soon as the server gets busy, it writes a corresponding
#   busy.html page into the project directory. When the task is
#   finished, the temporary table is removed and a redirecting
#   busy.html page is written. 
#
#   written by Claudio Lottaz (MPI-MG) in February-September 2004
#
################################################################################
#
#   Redraw prediction image
#

redraw.pred.img <- function(eval, data.path, aclass, show.graph, lab.len, sample.labels, 
                            f.red, f.green, f.blue, t.red, t.green, t.blue, 
                            minspec, minsens, maxsens, bin.thresh) {
  cat("    . redrawing prediction image...\n")
  write.busy(data.path, "Redrawing prediction images")
  show.graph <- ifelse(show.graph=="ON", T, F)
  sample.labels <- ifelse(sample.labels=="ON", T, F)
  filenamestem <- paste(data.path, "index", sep="")
  if (bin.thresh == "-") binthresh <- NULL
  else binthresh <- as.numeric(bin.thresh)
  time.stamp <- format(Sys.time(), "%j%H%M%S")

  file.remove(list.files(data.path, "index_pred_img[0-9]*.p", full.names=T))
  map <- image(eval@pred, outfile=filenamestem, aclass=aclass, 
               show.graph=show.graph,max.label.length=lab.len,sample.labels=sample.labels,
               col=stam.rgb.colors(r0=f.red, g0=f.green, b0=f.blue, 
                                   r1=t.red, g1=t.green, b1=t.blue), 
               minspec=minspec, minsens=minsens, maxsens=maxsens, bin.thresh=binthresh)
  file.rename(paste(data.path, "index_pred_img.png", sep=""),
              paste(data.path, "index_pred_img", time.stamp, ".png", sep=""))
  image(eval@pred, outfile=filenamestem, aclass=aclass, 
        show.graph=show.graph,max.label.length=lab.len, sample.labels=sample.labels, ps=T,
        col=stam.rgb.colors(r0=f.red, g0=f.green, b0=f.blue, 
                            r1=t.red, g1=t.green, b1=t.blue), 
        minspec=minspec, minsens=minsens, maxsens=maxsens, bin.thresh=binthresh)
  file.rename(paste(data.path, "index_pred_img.ps", sep=""),
              paste(data.path, "index_pred_img", time.stamp, ".ps", sep=""))

  lines <- readLines(paste(filenamestem, ".html", sep=""))
  lines[1] <- gsub("index_pred_img[0-9]*.p",
                   paste("index_pred_img", time.stamp, ".p", sep=""),
                   lines[1])
  lines[2] <- paste(MAP(name="pred_map", map),
                    stam.pred.img.form(filenamestem, aclass, 
                                       minspec, minsens, maxsens, bin.thresh,
                                       f.red, f.green, f.blue, t.red, t.green, t.blue,
                                       show.graph, lab.len, sample.labels), sep="")
  cat(lines, sep="\n", file=paste(filenamestem, ".html", sep=""))
}

################################################################################
#
#   Refit the model with new alpha
#

refit.model <- function(eval, data.path, aclass, delta, collapse.scnodes) {
  cat("    . refitting modes...\n")
  sink("tmp.log")
  write.busy(data.path, "Refitting model")
  collapse.scnodes <- ifelse(collapse.scnodes == "ON", TRUE, FALSE) 
  trainingset <- eval@cv@sample.labels
  fit <- stam.fit(eval@cv, eval@exprs[,trainingset], alpha=eval@fit@alpha, 
                  delta=delta, collapse.scnodes=collapse.scnodes)
  eval@fit <<- fit

  write.busy(data.path, "Predicting testset")
  pred <- stam.predict(fit, eval@exprs, eval@pred@cls, eval@testset)
  eval@pred <<- pred

  write.busy(data.path, "Replacing results")
  save(eval, file=paste(data.path, "eval.RData", sep=""), compress=T)
  filename <- paste(data.path, "index.html", sep="")
  file.remove(list.files(data.path, "index*", full.names=T))
  file.remove(list.files(data.path, "GO*", full.names=T))
  stam.writeHTML(eval, outfile=data.path, aclass = aclass)
  # avoid browser cache by using time stamps in figures
  time.stamp <- format(Sys.time(), "%j%H%M%S")
  for (f in list.files(data.path, "*.png", full.names=T))
    file.rename(f, sub(".png", paste(time.stamp, ".png", sep=""), f))
  for (f in list.files(data.path, "*.ps", full.names=T))
    file.rename(f, sub(".ps", paste(time.stamp, ".ps", sep=""), f))
  buffer <- readLines(filename)
  buffer[1] <- gsub(".png'>", paste(time.stamp,".png'>",sep=""), buffer[1])
  buffer[1] <- gsub('.ps">Down', paste(time.stamp,'.ps">Down',sep=""), buffer[1])
  cat(buffer, sep="\n", file=filename)
  sink()
  file.remove("tmp.log")
}

################################################################################
#
#   Auxiliaries
#

stam.env[["busy.buffer"]] <- ""
write.busy <- function(data.path, add.item = NULL) {
  if (stam.env[["busy.buffer"]] == "") {
    stam.env[["busy.buffer"]] <- paste("<HTML><HEAD><TITLE>Working...</TITLE>",
                         "<LINK REL='stylesheet' HREF='/includes/style.css' TYPE='text/css'>",
                         "</HEAD><BODY ONLOAD='setTimeout(\"document.location.reload(true)\", 2000)'><H1>Working...</H1></BODY></HTML>", sep="")
  }

  if (!is.null(add.item)) {
    stam.env[["busy.buffer"]] <- sub("\\.\\.\\.</LI>", paste("</LI><LI>",add.item,"...</LI>",sep=""), 
                        stam.env[["busy.buffer"]])
    stam.env[["busy.buffer"]] <- sub("</H1></BODY>", 
                        paste("</H1><UL><LI>", add.item, "...</LI></UL></BODY>", sep=""),
                        stam.env[["busy.buffer"]])
  }

  cat(stam.env[["busy.buffer"]], file=paste(data.path, "busy.html", sep=""))
}

write.redirect <- function(data.path, to.URL) {
  stam.env[["busy.buffer"]] <- ""
  filename <- paste(data.path, "busy.html", sep="")
  cat("<HTML><HEAD></HEAD><BODY ONLOAD='document.hiddenform.submit()'>",
      "<FORM NAME='hiddenform' action='/cgi-bin/stam/rmbusy.pl'>",
      "<INPUT TYPE=HIDDEN NAME=toURL VALUE='", to.URL, "'>",
      "<INPUT TYPE=HIDDEN NAME=datapath VALUE='", data.path, "'>",
      "</FORM></BODY></HTML>", sep="", file=filename)
}

################################################################################
#
#   Server loop
#

stam.serve <- function(tmp.path = NULL, cgi.path = NULL, cgi.url = NULL) {
  # modify installation if needed or required
  old.cgi.url <- stam.cgi.url
  if (!is.null(tmp.path)) assign("stam.tmp.path", tmp.path, parent.env(environment()))
  if (!is.null(cgi.path)) assign("stam.cgi.path", cgi.path, parent.env(environment()))
  if (!is.null(cgi.url))  assign("stam.cgi.url",  cgi.url,  parent.env(environment()))
  if (stam.tmp.path == "")
    stam.tmp.path <- readline(paste("Which is your temporary directory", 
                                    "with write permission for the WWW server?\n"))
  if (stam.cgi.path == "") 
    stam.cgi.path <- readline("Which is your CGI directory?\n")
  if (stam.cgi.url == "")
    stam.cgi.url <- readline("Which is the URL to the CGI directory?\n")
  if (stam.tmp.path == "") stop("stam.tmp.path not provided.")
  if (stam.cgi.path == "") stop("stam.cgi.path not provided.")
  if (stam.cgi.url == "") stop("stam.cgi.url not provided.")
  data.file <- paste(help(package="stam")$path, "/data/server.config.rda", sep="")
  save(stam.tmp.path, stam.cgi.path, stam.cgi.url, file=data.file)
  if (old.cgi.url != stam.cgi.url) 
    cat("The CGI URL has changed, make sure to regenerate the HTML pages\n",
        "to be worked with through the server.\n", sep="")
  
  # write stam.tmp.path into CGI scripts
  cgi.srcdir <- paste(library(help=stam)$path, "cgi", sep="/")
  for (f in list.files(cgi.srcdir, "*.pl", full.names=TRUE)) {
    flines <- readLines(f)
    flines <- gsub("<<<STAM.TEMP.PATH>>>", stam.tmp.path, flines)
    cgi.name <- gsub(cgi.srcdir, stam.cgi.path, f)
    writeLines(flines, cgi.name)
  }

  # start listening
  data.path = "";
  options(stam.write.forms=TRUE)
  cat(" - start listening '", stam.tmp.path, "'...\n", 
      "   Stop the server by creating '", stam.tmp.path, "/quit'.\n",
      "   Make sure you have written the HTML pages to be worked with through the\n",
      "   StAM server with 'options(stam.write.forms=TRUE)'\n", sep="")
  while (TRUE) {
    file.list <- list.files(stam.tmp.path, "quit", full.names=T)
    qfile <- grep("quit$", file.list)
    if (length(qfile) > 0) {
      file.remove(file.list[qfile])
      break
    }
    file.list <- list.files(stam.tmp.path, "*tab", full.names=T)
    for (curr.file in file.list) {
      cat(" - treating", curr.file, "...\n")
      tab <- read.table(curr.file, row.names=1, sep="\t")
      x <- as.character(tab[,1])
      names(x) <- rownames(tab)
      if (data.path != "") write.busy(x["data.path"])
      if (data.path != x["data.path"]) {
        cat("    . loading data...\n")
        write.busy(data.path, "Loading data")
        data.path <- x["data.path"]
        load(paste(data.path, "eval.RData", sep=""))
      }
      if (x["action"] == "redraw_pred_img") 
        redraw.pred.img(eval, data.path, x["aclass"], x["show.graph"], 
                        as.numeric(x["lab.len"]), x["sample.labels"], 
                        as.numeric(x["f.red"]), as.numeric(x["f.green"]), 
                        as.numeric(x["f.blue"]), as.numeric(x["t.red"]), 
                        as.numeric(x["t.green"]), as.numeric(x["t.blue"]), 
                        as.numeric(x["minspec"]), as.numeric(x["minsens"]), 
                        as.numeric(x["maxsens"]), x["bin.thresh"])
  
      if (x["action"] == "refit_model") 
        refit.model(eval,data.path,x["aclass"],as.numeric(x["delta"]),x["collapse_scnodes"])
      if (x["action"] == "remove_busy.html") {
        cat("    . removing ", data.path, "busy.html", "...\n", sep="")
        file.remove(paste(data.path, "busy.html", sep=""))
      } else write.redirect(data.path, x["to.URL"])
      cat("    . done\n")
      file.remove(curr.file)
    }
    Sys.sleep(1)
  }
}


#
#   End of File
#
################################################################################
